@@ -25,6 +25,26 @@ fn gen_napi_value_map_impl(name: &Ident, to_napi_val_impl: TokenStream) -> Token
       }
     }
 
+    impl napi::bindgen_prelude::TypeName for &#name {
+      fn type_name() -> &'static str {
+        #name_str
+      }
+
+      fn value_type() -> napi::ValueType {
+        napi::ValueType::Object
+      }
+    }
+
+    impl napi::bindgen_prelude::TypeName for &mut #name {
+      fn type_name() -> &'static str {
+        #name_str
+      }
+
+      fn value_type() -> napi::ValueType {
+        napi::ValueType::Object
+      }
+    }
+
     #to_napi_val_impl
 
     impl napi::bindgen_prelude::FromNapiRef for #name {
@@ -60,6 +80,36 @@ fn gen_napi_value_map_impl(name: &Ident, to_napi_val_impl: TokenStream) -> Token
         Ok(&mut *(wrapped_val as *mut #name))
       }
     }
+
+    impl napi::bindgen_prelude::FromNapiValue for &#name {
+      unsafe fn from_napi_value(
+        env: napi::bindgen_prelude::sys::napi_env,
+        napi_val: napi::bindgen_prelude::sys::napi_value
+      ) -> napi::bindgen_prelude::Result<Self> {
+        napi::bindgen_prelude::FromNapiRef::from_napi_ref(env, napi_val)
+      }
+    }
+
+    impl napi::bindgen_prelude::FromNapiValue for &mut #name {
+      unsafe fn from_napi_value(
+        env: napi::bindgen_prelude::sys::napi_env,
+        napi_val: napi::bindgen_prelude::sys::napi_value
+      ) -> napi::bindgen_prelude::Result<Self> {
+        napi::bindgen_prelude::FromNapiMutRef::from_napi_mut_ref(env, napi_val)
+      }
+    }
+
+    impl napi::NapiRaw for &#name {
+      unsafe fn raw(&self) -> napi::sys::napi_value {
+        unreachable!()
+      }
+    }
+
+    impl napi::NapiRaw for &mut #name {
+      unsafe fn raw(&self) -> napi::sys::napi_value {
+        unreachable!()
+      }
+    }
   }
 }
 
@@ -170,24 +220,36 @@ impl NapiStruct {
     quote! {
       impl napi::bindgen_prelude::ToNapiValue for #name {
         unsafe fn to_napi_value(
-          env: napi::bindgen_prelude::sys::napi_env, val: #name
-        ) -> napi::bindgen_prelude::Result<napi::bindgen_prelude::sys::napi_value> {
+          env: napi::sys::napi_env, val: #name
+        ) -> napi::Result<napi::bindgen_prelude::sys::napi_value> {
           if let Some(ctor_ref) = napi::bindgen_prelude::get_class_constructor(#js_name_str) {
             let mut ctor = std::ptr::null_mut();
 
-            napi::bindgen_prelude::check_status!(
-              napi::bindgen_prelude::sys::napi_get_reference_value(env, ctor_ref, &mut ctor),
+            napi::check_status!(
+              napi::sys::napi_get_reference_value(env, ctor_ref, &mut ctor),
               "Failed to get constructor of class `{}`",
               #js_name_str
             )?;
 
             let mut result = std::ptr::null_mut();
             napi::bindgen_prelude::___CALL_FROM_FACTORY.store(true, std::sync::atomic::Ordering::Relaxed);
-            napi::bindgen_prelude::check_status!(
-              napi::bindgen_prelude::sys::napi_new_instance(env, ctor, 0, std::ptr::null_mut(), &mut result),
+            napi::check_status!(
+              napi::sys::napi_new_instance(env, ctor, 0, std::ptr::null_mut(), &mut result),
               "Failed to construct class `{}`",
               #js_name_str
             )?;
+            napi::check_status!(
+              napi::sys::napi_wrap(
+                env,
+                result,
+                Box::into_raw(Box::new(val)) as *mut std::ffi::c_void,
+                Some(napi::bindgen_prelude::raw_finalize_unchecked::<#name>),
+                std::ptr::null_mut(),
+                std::ptr::null_mut(),
+              ),
+              "Failed to wrap native object of class `{}`",
+              #js_name_str
+            )?;
             napi::bindgen_prelude::___CALL_FROM_FACTORY.store(false, std::sync::atomic::Ordering::Relaxed);
             Ok(result)
           } else {
@@ -310,7 +372,12 @@ impl NapiStruct {
           if is_optional_field {
             obj_field_getters.push(quote! { let #ident: #ty = obj.get(#field_js_name)?; });
           } else {
-            obj_field_getters.push(quote! { let #ident: #ty = obj.get(#field_js_name)?.expect(&format!("Field {} should exist", #field_js_name)); });
+            obj_field_getters.push(quote! {
+              let #ident: #ty = obj.get(#field_js_name)?.ok_or_else(|| napi::bindgen_prelude::Error::new(
+                napi::bindgen_prelude::Status::InvalidArg,
+                format!("Missing field `{}`", #field_js_name),
+              ))?;
+            });
           }
         }
         syn::Member::Unnamed(i) => {
@@ -327,7 +394,12 @@ impl NapiStruct {
           if is_optional_field {
             obj_field_getters.push(quote! { let arg #i: #ty = obj.get(#field_js_name)?; });
           } else {
-            obj_field_getters.push(quote! { let arg #i: #ty = obj.get(#field_js_name)?.expect(&format!("Field {} should exist", #field_js_name)); });
+            obj_field_getters.push(quote! {
+              let arg #i: #ty = obj.get(#field_js_name)?.ok_or_else(|| napi::bindgen_prelude::Error::new(
+                napi::bindgen_prelude::Status::InvalidArg,
+                format!("Missing field `{}`", #field_js_name),
+              ))?;
+            });
           }
         }
       }
