@@ -76,7 +76,7 @@ fn escape_json(src: &str) -> String {
       c => {
         let encoded = c.encode_utf16(&mut utf16_buf);
         for utf16 in encoded {
-          write!(&mut escaped, "\\u{:04X}", utf16).unwrap();
+          write!(escaped, "\\u{:04X}", utf16).unwrap();
         }
       }
     }
@@ -169,6 +169,7 @@ static KNOWN_TYPES: Lazy<HashMap<&'static str, &'static str>> = Lazy::new(|| {
     ("BigUint64Array", "BigUint64Array"),
     ("DataView", "DataView"),
     ("Date", "Date"),
+    ("JsDate", "Date"),
     ("JsBuffer", "Buffer"),
     ("Buffer", "Buffer"),
     ("Vec", "Array<{}>"),
